' ********** Copyright 2015 Roku Corp.  All Rights Reserved. **********

' Component initialization
sub init()
    print "RokuBillingTask"
end sub


' onChange handler for "indexPurchase" field. Calls PurchaseProduct function automatically.
sub On_indexPurchase()
    if m.top.products <> invalid AND m.top.products.availForPurchase.list[m.top.indexPurchase] <> invalid then
        m.top.functionName = "PurchaseProduct"
        m.top.control = "RUN"
    end if
end sub


' Task function for getting shared user data. Sets "partialUserData" field.
sub GetPartialUserData()
    port = CreateObject("roMessagePort")
    channelStore = CreateObject("roChannelStore")
    channelStore.SetMessagePort(port)
    
    m.top.partialUserData = channelStore.GetPartialUserData("email")
end sub


' Task function for getting Roku Billing subscription products. Sets "products" field.
sub GetProducts()
    result = {
        availForPurchase : {
            list : []
            map  : {}
        }
        validPurchased : {
            list : []
            map  : {}
        }
    }
    allProducts = Helper_GetAllProducts()
    purchasedProducts = Helper_GetPurchasedProducts()
    
    datetime = CreateObject("roDateTime")
    utimeNow = datetime.AsSeconds()
    
    for each product in allProducts
        bAddToAvail = true
        for each purchase in purchasedProducts
            if purchase.code = product.code then
                bAddToAvail = false
                if purchase.expirationDate <> invalid then
                    datetime.FromISO8601String(purchase.expirationDate)
                    utimeExpire = datetime.AsSeconds()
                    if utimeExpire > utimeNow then
                        result.validPurchased.list.Push(purchase)
                        result.validPurchased.map[purchase.code] = purchase
                    end if
                end if
                exit for
            end if
        end for
        
        if bAddToAvail then
            result.availForPurchase.list.Push(product)
            result.availForPurchase.map[product.code] = product
        end if
    end for
    
    m.top.products = result
end sub


' Task function for purchasing Roku Billing product specified by "indexPurchase" field (see On_indexPurchase).
' Sets "purchaseResult" field.
sub PurchaseProduct()
    port = CreateObject("roMessagePort")
    channelStore = CreateObject("roChannelStore")
    channelStore.SetMessagePort(port)
    
    channelStore.ClearOrder()
    channelStore.SetOrder([{
        code : m.top.products.availForPurchase.list[m.top.indexPurchase].code
        qty  : 1
    }])
    channelStore.DoOrder()
    msg = invalid
    while type(msg) <> "roChannelStoreEvent"
        msg = Wait(0, port)
    end while
    
    result = {
        isSuccess : msg.isRequestSucceeded()
    }
    if msg.isRequestSucceeded() then
        response = msg.GetResponse()
        if response <> invalid AND response[0] <> invalid then
            result.Append(response[0])
        end if
    else if msg.isRequestFailed() then
        result.failureCode = msg.GetStatus()
        result.failureMessage = msg.GetStatusMessage()
    end if
    
    m.top.purchaseResult = result
end sub


' Helper function to get Roku Billing subscription products.
function Helper_GetAllProducts() as Object
    result = []
    
    port = CreateObject("roMessagePort")
    channelStore = CreateObject("roChannelStore")
    channelStore.SetMessagePort(port)
    
    channelStore.GetCatalog()
    msg = invalid
    while type(msg) <> "roChannelStoreEvent"
        msg = Wait(0, port)
    end while

    if msg.isRequestSucceeded() then
        response = msg.GetResponse()
        if response <> invalid then
            result = response
        end if
    end if
    
    return result
end function


' Helper function to get purchased Roku Billing products.
function Helper_GetPurchasedProducts()
    result = []
    
    port = CreateObject("roMessagePort")
    channelStore = CreateObject("roChannelStore")
    channelStore.SetMessagePort(port)
    
    channelStore.GetPurchases()
    msg = invalid
    while type(msg) <> "roChannelStoreEvent"
        msg = Wait(0, port)
    end while

    if msg.isRequestSucceeded() then
        response = msg.GetResponse()
        if response <> invalid then
            result = response
        end if
    end if
    
    return result
end function
