' ********** Copyright 2015 Roku Corp.  All Rights Reserved. **********

' Component initialization, setting default properties, configuring observers (handlers)
sub init()
    m.top.visible = false
    m.global.observeField("isSubscribe", "onGlobalSubscribeStatusChange")

    m.billing = m.top.FindNode("billing")
    m.billing.ObserveField("products", "On_billing_products")
    m.billing.ObserveField("purchaseResult", "On_billing_purchaseResult")

    m.subscriptionScreen = m.top.findNode("subscription")
    m.subscriptionScreen.regenerateActivationCodeButton.ObserveField("buttonSelected", "getActivationCode")

    m.authScreen = m.top.FindNode("authScreen")
    m.authScreen.buttonLogin.ObserveField("buttonSelected", "On_Login")
    m.authScreen.buttonSignup.ObserveField("buttonSelected", "On_Signup")
    m.authScreen.buttonSubscribe.ObserveField("buttonSelected", "On_Subscribe")
    m.authScreen.buttonLogout.ObserveField("buttonSelected", "On_Logout")
    updateUIs()

    m.loginAuthFlow = m.top.FindNode("loginAuthFlow")
    m.loginAuthFlow.dialogAuthFailed.title = "Login failed"
    m.loginAuthFlow.ObserveField("isAuthorized", "On_loginAuthFlow_isAuthorized")

    m.signupAuthFlow = m.top.FindNode("signupAuthFlow")
    m.signupAuthFlow.kbdialogEmail.title = "Enter the email address for new account creation"
    m.signupAuthFlow.kbdialogPassword.title = "Create the password"
    m.signupAuthFlow.dialogAuthFailed.title = "User account creation failed"
    m.signupAuthFlow.ObserveField("isAuthorized", "On_signupAuthFlow_isAuthorized")

    m.top.dialogSelectSub = CreateObject("roSGNode", "BackDialog")
    m.top.dialogSelectSub.title = "Subscription"
    m.top.dialogSelectSub.message = "Please select subscription type"
    m.top.dialogSelectSub.ObserveField("buttonSelected", "On_dialogSelectSub_buttonSelected")

    m.top.dialogNoSubToPurchase = CreateObject("roSGNode", "BackDialog")
    m.top.dialogNoSubToPurchase.title = "No available subscriptions to purchase"
    m.top.dialogNoSubToPurchase.message = "Please check your Roku account for previously purchased products."
    m.top.dialogNoSubToPurchase.buttons = ["OK"]
    m.top.dialogNoSubToPurchase.ObserveField("buttonSelected", "On_SubscriptionFailed")

    m.top.dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    m.top.dialogBillingNA.title = "Service unavailable"
    m.top.dialogBillingNA.message = "Subscription service is currently unavailable, please try again later."
    m.top.dialogBillingNA.buttons = ["OK"]
    m.top.dialogBillingNA.ObserveField("buttonSelected", "On_SubscriptionFailed")

    m.top.pdialogCheckSubs = CreateObject("roSGNode", "ProgressDialog")
    m.top.pdialogCheckSubs.title = "Checking subscriptions..."

    m.top.pdialogPurchase = CreateObject("roSGNode", "ProgressDialog")
    m.top.pdialogPurchase.title = "Purchasing subscription..."

    m.allDialogs = [
        m.loginAuthFlow.kbdialogEmail
        m.loginAuthFlow.kbdialogPassword
        m.loginAuthFlow.dialogErrEmail
        m.loginAuthFlow.dialogErrPassword
        m.loginAuthFlow.dialogAuthFailed

        m.signupAuthFlow.kbdialogEmail
        m.signupAuthFlow.kbdialogPassword
        m.signupAuthFlow.dialogErrEmail
        m.signupAuthFlow.dialogErrPassword
        m.signupAuthFlow.dialogTermsOfUse
        m.signupAuthFlow.dialogAuthFailed

        m.top.dialogSelectSub
        m.top.dialogNoSubToPurchase
        m.top.dialogBillingNA
    ]
end sub

sub onGlobalSubscribeStatusChange()
    updateUIs()
end sub


sub isAccountActivated()
    doRequest("checkIsActivated", sub(response)
           m.global.isSubscribe = response.status
    end sub, {baseURL: m.global.BASE_URL, email: getSavedEmail()})
end sub

sub isAccountRegistered()
    doRequest("isRegistered", sub(response)
          m.global.isLoggedIn = response.isregistered
          m.global.isSubscribe = response.isactivated
    end sub, {baseURL: m.global.BASE_URL, email: getSavedEmail()})
end sub 



'Update UIs visibility and focus
sub updateUIs()
    isSubscribed =  m.global.isSubscribe 
    m.subscriptionScreen.visible = false
    m.authScreen.buttonLogin.visible = NOT m.global.isLoggedIn AND NOT isSubscribed
    m.authScreen.buttonLogin.setFocus(NOT m.global.isLoggedIn AND NOT isSubscribed)
    m.authScreen.buttonSignup.visible = NOT m.global.isLoggedIn AND NOT isSubscribed
    m.authScreen.buttonSubscribe.visible = NOT isSubscribed AND m.global.isLoggedIn
    m.authScreen.buttonSubscribe.setFocus(NOT isSubscribed AND m.global.isLoggedIn)
    
    if (NOT m.global.isLoggedIn AND NOT isSubscribed)
        m.authScreen.labelText = "This is a demo. To use this application with your own playlists, please register an account using your device email and Activate Your Lifetime Premium Access"
    else if m.global.isLoggedIn AND NOT isSubscribed
        m.authScreen.labelText = "To use this application with your own playlists, please Activate Your Lifetime Premium Access"
    else
        m.authScreen.labelText = "You logged in to SageIPTV. Enjoy your channel!!!"
    end if

    m.authScreen.setUI = m.authScreen.buttonSubscribe.visible
    m.authScreen.buttonLogout.visible = NOT m.authScreen.buttonLogin.visible and m.top.visible
    m.authScreen.buttonLogout.setFocus(isSubscribed AND m.top.visible)
    m.global.isMenulocked = false
end sub

' onChange handler for "show" field
sub On_show()
    m.top.visible = m.top.show
    m.top.setFocus(m.top.show)
    isAccountRegistered()
    updateUIs()

    if m.top.show then
        if m.top.isAuthNeeded AND not m.top.isAuthorized then
            m.authScreen.show = m.top.show
        end if
    end if
end sub

' Hassan - do not try to to get activation code if token is empty
sub On_Subscribe()
    getActivationCode()
end sub

' onChange handler for "regexEmail" field
sub On_regexEmail()
    m.loginAuthFlow.regexEmail = m.top.regexEmail
    m.signupAuthFlow.regexEmail = m.top.regexEmail
end sub


' onChange handler for "regexPassword" field
sub On_regexPassword()
    m.loginAuthFlow.regexPassword = m.top.regexPassword
    m.signupAuthFlow.regexPassword = m.top.regexPassword
end sub


' onChange handler for "textTermsOfUse" field
sub On_textTermsOfUse()
    m.signupAuthFlow.dialogTermsOfUse.message = m.top.textTermsOfUse
end sub


' onChange handler for "dialogsConfig" field
sub On_dialogsConfig()
    if m.allDialogs = invalid OR m.top.dialogsConfig = invalid then
        return
    end if

    for each dialog in m.allDialogs
        dialog.SetFields(m.top.dialogsConfig)
    end for
end sub


' onChange handler for "dialogsButtonConfig" field
sub On_dialogsButtonConfig()
    if m.allDialogs = invalid OR m.top.dialogsButtonConfig = invalid then
        return
    end if

    for each dialog in m.allDialogs
        dialog.buttonGroup.SetFields(m.top.dialogsButtonConfig)
    end for
end sub


' Sets authorization selection screen buttons visibility
' @param isVisible [Boolean] True = buttons are visible; False = buttons are hidden
' @param isFocusButtonLogin [Boolean] True = set focus to login button; False = set focus to signup button. Default value is True. This parameter is not used if isVisible = False.
sub SetAuthScreenButtonsVisible(isVisible as Boolean, isFocusButtonLogin = true as Boolean)
    m.authScreen.buttonLogin.visible = isVisible
    m.authScreen.buttonSignup.visible = isVisible
    if isVisible then
        if isFocusButtonLogin then
            m.authScreen.buttonLogin.SetFocus(true)
        else
            m.authScreen.buttonSignup.SetFocus(true)
        end if
    end if
end sub

' Handler for processing logout flow button selection
sub On_Logout()
    'clear saved email when logging out
    saveToken("")
    saveEmail("")
    m.global.isSubscribe = false
    m.global.isLoggedIn = false
    updateUIs()
end sub


' Handler for processing login flow button selection
sub On_Login()
    ' login flow is started, hide auth selection screen buttons
    SetAuthScreenButtonsVisible(false)

    m.loginAuthFlow.kbdialogEmail.text = m.global.device_email
    m.loginAuthFlow.show = true
end sub


' Handler for processing signup flow button selection
sub On_Signup()
    m.signupAuthFlow.kbdialogEmail.text = m.global.device_email
    m.signupAuthFlow.show = true
end sub

' Handler for processing authorization by login API (also invoked if only Roku Billing subscription used)
sub On_loginAuthFlow_isAuthorized()
    'm.top.isAuthorized = m.loginAuthFlow.isAuthorized
    m.top.isAfterLoginAuthFlow = true
end sub


' Handler for processing authorization by signup API
sub On_signupAuthFlow_isAuthorized()
    'm.top.isAuthorized = m.signupAuthFlow.isAuthorized
    m.top.isAfterLoginAuthFlow = false
end sub


' Handler for processing completion of authorization flow using login/signup API
sub On_isAfterLoginAuthFlow()
    if GetParentScene() = invalid then
        return
    end if

    'if m.top.isAuthorized
        saveToken(m.global.token)
        saveEmail(m.global.device_email)
    'end if

    m.global.isSubscribe = false
    m.top.isSubscribed = false
    onCheckSubsriptionStatusAfterLogin_SignUp()
end sub


' Handler for processing Roku Billing subscription products returned from Billing Task
sub On_billing_products()
    if GetParentScene() = invalid then
        return
    end if

    if m.bAfterAuthLogin <> invalid AND m.bAfterAuthLogin AND m.billing.products.validPurchased.list.Count() > 0 then
        m.top.isSubscribed = true
    else if m.billing.products.availForPurchase.list.Count() > 0 then
        subscriptions = []
        for each item in m.billing.products.availForPurchase.list
            subscriptions.Push(item.name + " " + item.cost)
        end for
        m.top.dialogSelectSub.buttons = subscriptions
        m.parentScene.dialog = m.top.dialogSelectSub
    else if m.billing.products.validPurchased.list.Count() > 0
        m.parentScene.dialog = m.top.dialogNoSubToPurchase
    else
        m.parentScene.dialog = m.top.dialogBillingNA
    end if
end sub


' Handler for processing subscription selected from subscription selection dialog
sub On_dialogSelectSub_buttonSelected()
    if GetParentScene() = invalid then
        return
    end if

    if m.top.dialogSelectSub.buttonSelected < 0 then
        On_SubscriptionFailed()
    else
        m.parentScene.dialog = m.top.pdialogPurchase
        m.billing.indexPurchase = m.top.dialogSelectSub.buttonSelected
    end if

    updateUIs()
end sub


' Handler for processing subscription failure
sub On_SubscriptionFailed()
    m.top.isSubscribed = false
    updateUIs()
end sub


' Handler for processing subscription product purchase result
sub On_billing_purchaseResult()
    if m.billing.purchaseResult = invalid OR m.billing.purchaseResult.isSuccess = invalid then
        m.top.isSubscribed = false
        return
    end if

    m.top.purchaseID = m.billing.purchaseResult.purchaseID
    m.top.isSubscribed = m.billing.purchaseResult.isSuccess
    updateUIs()
end sub

'****************** code started by gaurav'

'********** below method is used for hitting login up api which check the user is valid or not'
sub hitLoginApi()
    data = m.top.loginUserData
    data.baseURL = m.global.BASE_URL
    doRequest("checkLogin", sub(response)
        m.global.isLoggedIn = response.status
        m.global.Token = response.token
        m.top.isLoginAPISuccess = response.status
        m.authScreen.buttonSubscribe.visible = response.status AND NOT m.top.isSubscribed
    end sub, data)
end sub

'********** below method is used for hitting signup up api at the time of creating new user'
sub hitSignUpApi()
    data = m.top.signupUserData
    data.baseUrl = m.global.BASE_URL
    doRequest("apiUserRegistration", sub(response)
        m.global.Token = response.token
        m.global.isLoggedIn = (response.status = "success")
        m.top.isSignupAPISuccess = (response.status = "success")
        m.signUpAccountTask = invalid
    end sub, data)
end sub

'****************** code ends here'

'below method is used for fetching activation code for subscription'
sub getActivationCode()
    data = {}
    data.baseURL = m.global.BASE_URL
    data.token = m.global.token
    doRequest("fetchActivationCode", sub(response)
          On_activationCodeResponse(response)
    end sub, data)
end sub

'below method is used for handling activation code api response for subscription'
sub On_activationCodeResponse(resp)
    if resp.status = "success"
       m.subscriptionScreen.activationCode = resp.code
       m.subscriptionScreen.visible = true
       startSubscriptionCheckTimer()
    else
       ? "something went wrong while fetching activation code"
    end if
end sub

sub startSubscriptionCheckTimer()
    if m.subscriptionTimer = invalid
       m.subscriptionTimer = CreateObject("roSGNode","Timer")
       m.subscriptionTimer.duration = 5
       m.subscriptionTimer.repeat = False
       m.subscriptionTimer.observeField("fire","onCheckSubsriptionStatus")
       m.subscriptionTimer.control = "start"
    end if
end sub

'below method will be called every after 5 seconds to check the subscription status after pressing subscribe button i.e to check user has entered code or not'
sub onCheckSubsriptionStatus()
    m.subscriptionTimer = invalid
    doRequest("checkSubscription", sub(response)
        if NOT response.status
           startSubscriptionCheckTimer()
        else
           m.loginAuthFlow.isAuthorized = response.status
           m.top.isAuthorized = response.status
           m.global.isSubscribe = response.status
           m.top.isSubscribed = response.status
           updateUIs()
        end if
    end sub, {baseURL: m.global.BASE_URL, token: m.global.token})
end sub

'below method will be called only once after login or after signup to check the user subscription status'
sub onCheckSubsriptionStatusAfterLogin_SignUp()
    doRequest("checkSubscription", sub(response)
           m.top.isAuthorized = response.status
           m.global.isSubscribe = response.status
           m.top.isSubscribed = response.status
           updateUIs()
    end sub, {baseURL: m.global.BASE_URL, token: m.global.token})
end sub
