function init()
    m.indexButtonGo = 0
    m.indexButtonBack = 1
    m.ADD_NEW_BUTTON_INDEX = 0

    m.loadingIndicator = CreateObject("roSGNode", "ProgressDialog")
    m.loadingIndicator.title = "Please wait..."

    m.lists = m.top.findNode("lists")
    
    
    
    m.lists.observeField("itemSelected", "OnListSelected")
    m.actions = m.top.findNode("actions")
    m.actions.observeField("itemSelected", "OnActionSelected")

    m.kbdialogLabel = getDialogLabel()
    m.kbdialogUrl = getDialogUrl()

end function

function getDialogLabel() as Object
    kbdialogLabel = CreateObject("roSGNode", "BackKeyboardDialog")
    kbdialogLabel.id = "label"
    kbdialogLabel.title = "Please enter the playlist label"
    kbdialogLabel.text = ""
    kbdialogLabel.buttons = ["Continue", "Back"]
    kbdialogLabel.ObserveField("buttonSelected", "onDialogButtonSelected")
    kbdialogLabel.observeField("wasClosed", "oDialogWasClosed")

    return kbdialogLabel
end function

function getDialogUrl() as Object
    kbdialogUrl = CreateObject("roSGNode", "BackKeyboardDialog")
    kbdialogUrl.id = "url"
    kbdialogUrl.title = "Please enter the playlist url"
    kbdialogUrl.text = ""
    kbdialogUrl.buttons = ["Submit", "Back"]
    kbdialogUrl.ObserveField("buttonSelected", "onDialogButtonSelected")
    kbdialogUrl.observeField("wasClosed", "oDialogWasClosed")

    return kbdialogUrl
end function

function loadLists(params = invalid as Object) as Boolean
    showLoading()
    doRequest("getManageLists", sub(response)
        hideLoading()
        m.lists.content = response.content
    end sub , { baseURL: m.global.BASE_URL, email: getEmail() })

    return true
end function

sub submitNewList(label as String, url as String)
    showLoading()
    doRequest("addNewList", sub(response)
        hideLoading()
        loadLists()
        resetDialogsText()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        label: label.Escape()
        encodedURL: url.Escape()
    })
end sub

sub updateList(id as String, label as String, url as String)
    showLoading()
    doRequest("updateList", sub(response)
        hideLoading()
        loadLists()
        resetDialogsText()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
        label: label.Escape()
        encodedURL: url.Escape()
    })
end sub


sub refreshSelectedList(id as String)
    showLoading()
    doRequest("refreshList", sub(response)
        hideLoading()
        loadLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
    })
end sub

sub deleteSelectedList(id as String)
    showLoading()
    doRequest("deleteList", sub(response)
        hideLoading()
        loadLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
    })
end sub

sub showLoading()
    m.top.GetParent().dialog = m.loadingIndicator
end sub

sub hideLoading()
    if m.top.getScene().dialog <> invalid
        m.top.getScene().dialog.close = true
    end if
end sub

sub resetDialogsText()
    m.kbdialogLabel = getDialogLabel()
    m.kbdialogUrl = getDialogUrl()
end sub

sub OnListSelected()
    m.actions.setFocus(true)
end sub

sub onDialogButtonSelected(event as Object)
    node = event.getRoSGNode()
    buttonIndex = event.getData()

    if node.id = "label"
        if (buttonIndex = m.indexButtonGo)
            m.parentScene.dialog = m.kbdialogUrl
        else
            m.parentScene.dialog.close = true
            resetDialogsText()
        end if
    else if node.id = "url"
        m.parentScene.dialog.close = true
        if (buttonIndex = m.indexButtonGo)
            if (m.actions.itemFocused = m.ADD_NEW_BUTTON_INDEX)
                submitNewList(m.kbdialogLabel.text, m.kbdialogUrl.text)
            else
                content = getCurrentSelectedListContent()
                if (content <> invalid)
                    updateList(content.id, m.kbdialogLabel.text, m.kbdialogUrl.text)
                end if
            end if
        else
            resetDialogsText()
        end if
    end if
end sub

sub OnActionSelected(event as Object)
    if GetParentScene() = invalid then
        return
    end if

    if not m.global.isSubscribe
        showNonSubscribeNotification()
        return
    end if

    action = event.getData()

    if action = 0
        m.parentScene.dialog = m.kbdialogLabel
    else if action = 1
        content = getCurrentSelectedListContent()
        if (content <> invalid)
            m.kbdialogLabel.text = content.label
            m.kbdialogUrl.text = content.url
            m.parentScene.dialog = m.kbdialogLabel
        end if
    else if action = 2
        content = getCurrentSelectedListContent()
        if (content <> invalid)
           refreshSelectedList(content.id)
        end if
    else if action = 3
        content = getCurrentSelectedListContent()

        if (content <> invalid)
            deleteSelectedList(content.id)
        end if
    end if
end sub

sub showNonSubscribeNotification()
    dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    dialogBillingNA.title = "Activate Your Lifetime Premium Access!"
    dialogBillingNA.message = "To manage your playlists, you have to Activate Your Lifetime Premium Access. Under info click <Activate> button and follow the steps on the screen if you have already signed up."
    dialogBillingNA.buttons = ["OK"]
    dialogBillingNA.ObserveField("buttonSelected", "OnNoSubscriptionAlertClick")

    m.parentScene.dialog = dialogBillingNA
end sub

sub showEmptyListsMessage()
    dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    dialogBillingNA.title = "Empty Playlist"
    dialogBillingNA.message = "Please select a playlist!"
    dialogBillingNA.buttons = ["OK"]
    dialogBillingNA.ObserveField("buttonSelected", "OnNoSubscriptionAlertClick")

    m.parentScene.dialog = dialogBillingNA
end sub

sub OnNoSubscriptionAlertClick()
    m.parentScene.dialog.close = true
end sub

function getCurrentSelectedListContent() as Object
    selectedIndex = m.lists.itemFocused

    if (m.lists.content = invalid)
        showEmptyListsMessage()
        return invalid
    end if
    return m.lists.content.getChild(selectedIndex)
end function

sub onFocusChange(event as Object)
    focus = event.getData()

    m.top.findNode("lists").setFocus(focus)
end sub

function onKeyEvent(key as String, press as Boolean) as Boolean
    if press
        if key = "left" and m.actions.hasFocus()
            m.lists.setFocus(true)
            print "empty list"
            return true
        else if key = "right" and m.lists.hasFocus()
            m.actions.setFocus(true)
            return true
        else if key = "down" and m.lists.content = invalid
          m.actions.setFocus(true)
        end if
    end if
    return false
end function
