function doRequest(name as String, onResultCallback as Dynamic, params = invalid as Object, onErrorCallback = invalid as Dynamic) as Object 
    requestObject = createObject("roSGNode", "RequestObject")
    
    requestObject.name = name
    if params <> invalid
        requestObject.params = params
    end if
    
    if m.onResultFunctions = invalid
        m.onResultFunctions = {}
    end if
    
    if m.onErrorFunctions = invalid
        m.onErrorFunctions = {}
    end if
    
    uniqueId = getUniqueId()
    requestObject.id = uniqueId
    
    if type(onResultCallback) = "Function"
	    m.onResultFunctions.addReplace(uniqueId, onResultCallback)
    end if
    
    if type(onErrorCallback) = "Function"
	    m.onErrorFunctions.addReplace(uniqueId, onErrorCallback)
    end if
    
    requestObject.observeField("result", "onContentRequestResult")
    requestObject.observeField("error", "onContentRequestError")
    
	requestTask = createObject("roSGNode", "RequestTask")
	requestTask.observeField("state", "onRequestTaskStateChange")
	requestTask.setFields({
		id: uniqueId
		request: requestObject
		control: "run"
	})
	if m.requestTasks = invalid
		m.requestTasks = {}
	end if
	m.requestTasks.addReplace(uniqueId, requestTask)
	
	return requestTask
end function


sub onRequestTaskStateChange(event as Object)
	if event.getData() = "stop"
		m.requestTasks.delete(event.getNode())
	end if
end sub


sub onContentRequestResult(event as Object)
    requestId = event.getNode()
    onResultFunction = m.onResultFunctions.lookup(requestId)
    if onResultFunction <> invalid
        onResultFunction(event.getData())
        m.onResultFunctions.delete(requestId)
    end if
end sub


sub onContentRequestError(event as Object)
    requestId = event.getNode()
    onErrorFunction = m.onErrorFunctions.lookup(requestId)
    if onErrorFunction <> invalid
        onErrorFunction(event.getData())
        m.onErrorFunctions.delete(requestId)
    end if
end sub


function getUniqueId() as String
	if m.uniqueId = invalid
		m.uniqueId = 0
	end if
	
	m.uniqueId++
	
	return m.uniqueId.toStr()
end function


' Converts Associative Array into ContentNode tree
'@param data Associative Array object, which will be mapped into ContentNode, children should be put into childrenArr field, every child will be processed the same way.
function AAToNodeTree(data as Object) as Object
    result = invalid
    
    dataType = type(data)
    
    if dataType = "roSGNode"
        result = data
    else if dataType = "roAssociativeArray"
        fieldsToSet = {}
        
        fieldsToSet.append(data)
        children = fieldsToSet.childrenArr
        fieldsToSet.delete("childrenArr")
        
        subtype = data.subtype
        if not type(subtype) = "String" then subtype = "ContentNode"
        
        result = AAToContentNode(fieldsToSet, subtype)
        if result <> invalid and type(children) = "roArray"
            contentNode = invalid
            childrenArr = []
            for each child in children
                contentNode = AAToNodeTree(child)
                if contentNode <> invalid then childrenArr.push(contentNode)
            end for
            result.appendChildren(childrenArr) 
        end if
    end if
        
    return result
end function


' Force sets fields to a given node. If node doesn't have a field, it adds it and then sets
' @param node roSGNode, to which you want to set fields
' @param fieldsToSet associative array of field names and values to be set 
sub forceSetFields(node as Object, fieldsToSet as Object)
	if not type(node) = "roSGNode" or not type(fieldsToSet) = "roAssociativeArray" then return
	
	existingFields = {}
    newFields = {}
    
      'AA of node read-only fields for filtering'
    fieldsFilterAA = {
        focusedChild    :   "focusedChild"
        change          :   "change"
        metadata        :   "metadata"
    }

    for each field in fieldsToSet
        if node.hasField(field)
            if NOT fieldsFilterAA.doesExist(field) then existingFields[field] = fieldsToSet[field]
        else
            newFields[field] = fieldsToSet[field]
        end if
    end for
    
    node.setFields(existingFields)
    node.addFields(newFields)
end sub


function AAToContentNode(inputAA = {} as Object, nodeType = "ContentNode" as String)
    item = createObject("roSGNode", nodeType)
    
    existingFields = {}
    newFields = {}
    
      'AA of node read-only fields for filtering'
    fieldsFilterAA = {
        focusedChild    :   "focusedChild"
        change          :   "change"
        metadata        :   "metadata"
    }

    for each field in inputAA
        if item.hasField(field)
            if NOT fieldsFilterAA.doesExist(field) then existingFields[field] = inputAA[field]
        else
            newFields[field] = inputAA[field]
        end if
    end for
    
    item.setFields(existingFields)
    item.addFields(newFields)
    
    return item
end function


'*********** below method is used for creating username by replacing . with _ '
function createUsername(email as String)
    usernameArr = email.split(".")
    finalUserName = ""
    if usernameArr <> invalid and usernameArr.Count() > 0
          for i = 0 to usernameArr.Count()-1 Step+1
              if i = 0
                  finalUserName = usernameArr[i] + "_"
              else
                  finalUserName = finalUserName + "_" + usernameArr[i]
              end if
          end for
    end if
    return finalUserName
end function

function getEmail() as String
    if (m.global.isLoggedIn)
        return m.global.token
    else
        return "default@sageiptv.com"
    end if    
end function


Function getJson(xfer as Object) As Dynamic
    ret = xfer.GetToString()
    if ret = ""
        json = invalid
     else
       json = ParseJson(ret)
    endif
    return json
End Function

function toString(value as Dynamic) as String
	if value = invalid
		return ""
	else
		return value.toStr()
	end if
end function

'used for delayed execution of function
'@param func [String] global function name
'@param delay [Float] delay interval
Sub executeAfterDelay(func as String, delay = 0.5 as float)
    m.timer = CreateObject("roSGNode","Timer")
    m.timer.duration = delay
    m.timer.ObserveField("fire",func)
    m.timer.control = "start"
End Sub

Function HttpEncode(str As String) As String
    o = CreateObject("roUrlTransfer")
    return o.Escape(str)
End Function


Function StripHTMLTags(str As String) As String
    regex = CreateObject("roRegex", "<(.|\n)*?>", "i")
    Return regex.ReplaceAll(str, "")
End Function

function saveEmail(email as String) as Object
    section = CreateObject("roRegistrySection", "Authentication")
    section.Write("LOGGED_EMAIL", email)
    section.Flush()
end function

function saveToken(token as String) as Object
    section = CreateObject("roRegistrySection", "Authentication")
    section.Write("LOGGED_TOKEN", token)
    section.Flush()
end function


function getSavedEmail() as String
    section = CreateObject("roRegistrySection", "Authentication")
    email = ""
    if section.Exists("LOGGED_EMAIL") then
        email = section.Read("LOGGED_EMAIL")
    end if
    return email
end function

function getSavedToken() as String
    section = CreateObject("roRegistrySection", "Authentication")
    token = ""
    if section.Exists("LOGGED_TOKEN") then
        token = section.Read("LOGGED_TOKEN")
    end if
    return token
end function
